""" A module to read a DART dump file.
"""

import os, sys, re

#from Euclid import Euclid_TC_codes, Euclid_TM_codes
import Euclid_TC_codes, Euclid_TM_codes

import datetime, time


if sys.version_info[0] < 3:
   from ConfigParser import RawConfigParser
else:
   from configparser import RawConfigParser


""" This class provides an enum like constant for decoding the type of line read from the file. """
class LineType:
#    TMPAR, TC, Event, TM_PACKET, HK_PACKET, Empty, Err = range(7)
    TMPAR, TC, Event, TM_PACKET, HK_PACKET, RT_FAST, RT_SLOW, RT_SUPERFAST, ST_SLOW, ST_SUPERFAST, Empty, Err = range(12)

#logTime = ''


reject_pattern   = re.compile("name")    # TM, TMPAR
reject_pattern1  = re.compile("logTime") # EV
search_pattern   = re.compile("^TMPAR")
tmpar_pattern    = re.compile("^TMPAR")
event_pattern    = re.compile("^EV")
packet_pattern   = re.compile("^TM,")
#command_pattern  = re.compile("^Commands")
command_pattern  = re.compile("^TC,")

hk_pkt_mib_id = {'310133001' : LineType.RT_SLOW,
                 '310133002' : LineType.ST_SLOW,
                 '310133003' : LineType.RT_FAST,
                 '310133004' : LineType.RT_SUPERFAST,
                 '310133005' : LineType.ST_SUPERFAST
}

""" This class implements the Dump file reader. """
class EchoReader(object):


    def __init__(self,config_type):
        super(EchoReader, self).__init__()

        self._filepath = ""
        #self._fileId = 0
        self._fileId = None
        self._line = ""
        self._error = 0
        ###self._eof = 0
        self._eof = False
        ###self._stop = 0

        self._fileSize = 0
        self._currentPos = 0

        self._logTime = ''
        self.config_type = config_type
        print("EchoReader ConfigType: " + str(config_type))
        # Instantiate the configuration file parser
        self.config = RawConfigParser()
        # Make it case-sensitive for parameter names
        self.config.optionxform = str
        # Open the configuration file
        self.config.read ("EchoReader.ini")
        testparam = self.config.getint(self.config_type, 'Test_Param')
        print("TestParam: " + str(testparam))

        self.ev_obt         = self.config.getint(self.config_type,'EV_OBT')
        self.ev_entryno     = self.config.getint(self.config_type,'EV_ENTRYNO')
        self.ev_severity    = self.config.getint(self.config_type,'EV_SEVERITY')
        self.ev_source      = self.config.getint(self.config_type,'EV_SOURCE')
        self.ev_workstation = self.config.getint(self.config_type,'EV_WORKSTATION')
        self.ev_message     = self.config.getint(self.config_type,'EV_MESSAGE')

        self.tmpar_name     = self.config.getint(self.config_type, 'TMPAR_NAME')
        self.tmpar_rawvalue = self.config.getint(self.config_type, 'TMPAR_RAWVALUE')
        self.tmpar_engvalue = self.config.getint(self.config_type, 'TMPAR_ENGVALUE')
        self.tmpar_state    = self.config.getint(self.config_type, 'TMPAR_STATE')
        self.tmpar_obt      = self.config.getint(self.config_type, 'TMPAR_OBT')

        self.tmpar_obt_conv = self.config[self.config_type]['OBT_CONV_STR']

        self.tm_obt     = self.config.getint(self.config_type, 'TM_OBT')
        self.tm_apid    = self.config.getint(self.config_type, 'TM_APID')
        self.tm_apid2   = self.config.getint(self.config_type, 'TM_APID2')
        self.tm_type    = self.config.getint(self.config_type, 'TM_TYPE')
        self.tm_subtype = self.config.getint(self.config_type, 'TM_SUBTYPE')
        self.tm_warn    = self.config.getint(self.config_type, 'TM_WARN')
        self.tm_logtime = self.config.getint(self.config_type, 'TM_LOGTIME')
        self.tm_sid     = self.config.getint(self.config_type, 'TM_SID')

        self.tc_obt     = self.config.getint(self.config_type, 'TC_OBT')
        self.tc_apid    = self.config.getint(self.config_type, 'TC_APID')
        self.tc_apid2   = self.config.getint(self.config_type, 'TC_APID2')
        self.tc_type    = self.config.getint(self.config_type, 'TC_TYPE')
        self.tc_subtype = self.config.getint(self.config_type, 'TC_SUBTYPE')
        self.tc_warn    = self.config.getint(self.config_type, 'TC_WARN')
        self.tc_logtime = self.config.getint(self.config_type, 'TC_LOGTIME')


    ## Opens a DART dump file for reading.
    #  @param filepath A string containing the full path to a dump file.
    #  Read in and bin the first 3 header description lines
    def open_for_reading(self, filepath):
        """ Opens A DARTS dump file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        print("EchoReader Opening %s\n" % filepath)
        try:
            self._fileId = open(filepath, 'r')
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            for i in range(0, 3):
                foo = self.next_line()


    # Just in case the header lines are different from the tsv file
    def open_csv_file(self, filepath, mode):
        """ Opens A DARTS dump file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        print(f"Opening {filepath}")
        try:
            self._fileId = open(filepath, 'r')
            print(f"Opened {filepath} for reading ok")

            self.get_size()
            self._currentPos = self._fileId.tell()
            print(f'open_csv_file: currentPos = {self._currentPos}, file size = {self._fileSize}')
            if self._currentPos < self._fileSize:
               l = self.next_line()
               print(f'open_csv_file: read line {l} at open file')

            if mode == 'realtime':
               # seek to the end of file
                self._fileId.seek(0, 2)
            ###self._stop = 0
            ###self._eof = 0
            self._eof = False
            self._fileId.seek(0, os.SEEK_END)
            self._currentPos = self._fileId.tell()
            print(f'csv_file_open: after seek to end of file, currentPos = {self._currentPos}')
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            # Can't do this if interactive...
            #for i in range(0, 3):
                #foo = self.next_line()


    def close_input(self):
        """ Closes access to the dump file. """
        if self._fileId == None:
            return
        try:
            self._fileId.close()
            ###self._stop = 1
        except Exception as e:
            print("Error closing file")


    def stop(self):
        """ Module wide marker to implement the stop reading file function."""
        ###self._stop = 1
        pass

    def get_pointer(self):
        return self._fileId.tell()


    def seek_tolocation(self,location):
        return self._fileId.seek(location)


    def get_size(self):
       self._fileSize = os.path.getsize(self._filepath)


    def next_line(self):
        """ Return the next line from the dump file."""
        #self._error = self._stop
        self._error = 0
#        if self._stop:
#            return self._stop
        try:
            self._line = self._fileId.readline().rstrip('\n')	# Works but screws up text field scrolling...
####            print("next_line: " + self._line)
        except Exception as e:
            print("Exception thrown reading file")
            self._line = "ERROR"
            self._error = 100
        finally:
            return self._error


#Events\tentryNo\tseverity\tsource\tworkstation\tobt\tlogTime\tmessage
#TM Parameters\tpk\tname\trawValue\tengValue\tmonState\tgenerationTime\trawValidity\tengValidity\tid\treceivedTime\tbitOffset\tbitSize\tsample\tcurve\tlimits\talert
#Commands\tname\tentryNo\tpktOffset\tpktSize\tid\tapid\tssc\ttype\tstype\tcompleted\ttimeTagged\tstageHistory\tmapid\tobt\ttrx\tlastUpdateTime\tparams

    def decode_line(self):
        """ Return a string built from decoding the input line from the dump file."""
        search_pattern = re.compile("^TM Parameters")
        words = self._line.split('\t')
        if search_pattern.search(self._line):
            # TM, Generation time, name, rawValue, engValue, monState
            return (LineType.TM, (words[6],words[2],words[3],words[4],words[5]))
        search_pattern = re.compile("^Events")
        if search_pattern.search(self._line):
            # Event, Generation time, log time, entryNo, severity, source, workstation, message...
            return (LineType.Event, words[6] + " Evt: " + words[1] + "/" + words[2] + " " + words[3] + " " +  words[4] + " ".join(words[7:]))
        search_pattern = re.compile("^Commands")
        if search_pattern.search(self._line):
            type_subtype = self.decode_tc_type_subtype(words[8], words[9])
            # Command, lastupdateTime, ssc?
            return (LineType.TC, words[15] + " TC: " + words[6] + " (0x" + str(int(words[6], base=16)) + ") " + type_subtype)

        ###self._eof = 1
        self._eof = True
        return (LineType.Err, "Unknown line")


# TM,trx,id,spid,name,apid,ssc,type,stype,pi1,vc,obt,obtOk,crcFlag,crcOk,alarm,warning
# TMPAR,name,rawValue,engValue,monState,sample,packetRef
# EV,logTime,entryNo,severity,source,workstation,message

    def decode_csv_line(self):
        """ Return a string built from decoding the input line from the dump file."""

        words = self._line.split(',')
        #print(f"decode_csv_line: {words}")

        if (reject_pattern.search(self._line)) or (reject_pattern1.search(self._line)):
            #self._eof = 1
            print("REJECT")
            return (LineType.Err, "Unknown line")

        # TMPAR
        #     0   1         2                    3   4   5            6   7   8   9 10  11   12    13
        # TMPAR, \N, VOGT2805, VIS_OBS-G_SW_TASK01, \N, \N,        PEND, OK, \N, \N, 3, 25, 4.0, \N,   2023-06-23T10:00:43.382288, 2023-06-23T10:00:41.374594, 2023-06-23T10:00:41.374594 (Enumerated)
        # TMPAR, \N, VDGT2727, VIS_CDPU_VDIG_CURR,  \N, 0.1946462, \N,   OK, \N, \N, 3, 25, 4.0, 1722, 2023-06-23T10:00:53.303257, 2023-06-23T10:00:51.376336, 2023-06-23T10:00:51.376336 (engineering val)
        # TMPAR, \N, VDGT2753, VIS_CDPU_EDAC_CNT1,  \N, 0.0,       \N,   OK, \N, \N, 3, 25, 4.0, 0,    2023-06-23T10:01:13.427665, 2023-06-23T10:01:11.388347, 2023-06-23T10:01:11.388347 (counter)
        if tmpar_pattern.search(self._line):
           #print('TMPAR')
           logTime = '0'

            #use log time that came from TM,
           if(len(self._logTime) > 0) :
              logTime = self._logTime

            #If it is available and the TMPAR has a OBT time then use it as default
           if self.tmpar_obt > -1 :
              dTest = datetime.datetime.strptime(words[self.tmpar_obt], self.tmpar_obt_conv)
              logTimeTest = dTest.strftime("%Y-%m-%dT%H:%M:%S.%f") 
              logTime = str(logTimeTest)
               #logTime = words[tmpar_obt]
              
           if self.tmpar_state == -1:
              return (LineType.TMPAR, (words[self.tmpar_obt], words[self.tmpar_name],words[self.tmpar_rawvalue],words[self.tmpar_engvalue],'OK'))
           else:
              return (LineType.TMPAR, (words[self.tmpar_obt], words[self.tmpar_name],words[self.tmpar_rawvalue],words[self.tmpar_engvalue],words[self.tmpar_state],words[self.tmpar_engvalue + 1]))

        # Event
        if event_pattern.search(self._line):
#           pass

#           if len(self._logTime) == 0 and self.ev_obt > -1 :
#              self._logTime = self.ev_obt              

#           return (LineType.Event, words[self.ev_obt] + " Evt: " + words[self.ev_entryno] + "/" + words[self.ev_severity] + " " + words[self.ev_source] + " " +  words[self.ev_workstation] + " ".join(words[self.ev_message]))
           return (LineType.Event, words[self.ev_obt] + " Evt: " + words[self.ev_entryno] + "/" + words[self.ev_severity] + " " + words[self.ev_source] + " " +  words[self.ev_workstation] + " ".join(words[self.ev_message]))

           
        # Command
        if command_pattern.search(self._line):
           #print("TC PACKET")
           if "\\N" in self._line: # or "eucdcl4" in self._line:
              return (LineType.Empty, "Not for us")
           type_subtype = self.decode_tc_type_subtype(words[self.tc_type], words[self.tc_subtype]) 
           return (LineType.TC, (words[self.tc_obt], words[self.tc_apid], type_subtype, words[self.tc_warn]))
        
        # TM packet
        if packet_pattern.search(self._line):
           #print(f"TM: {self._line}")
           typ  = int(words[self.tm_type])
           styp = int(words[self.tm_subtype])
           type_subtype = self.decode_tm_type_subtype(typ, styp)
           self._logTime = words[self.tm_logtime]
           pkt_freq = words[3]
           sid = words[self.tm_sid]
           if sid in hk_pkt_mib_id:
              ptype = hk_pkt_mib_id[sid]
           else:
              ptype = LineType.Empty
           #print(f'ptype = {ptype}')
           packet_type = LineType.HK_PACKET if ((typ == 3) and (styp == 25)) else LineType.TM_PACKET
           return (packet_type, (words[self.tm_obt], words[self.tm_apid], type_subtype, words[self.tm_warn], ptype))
        
        ####################################self._eof = 1
        return (LineType.Empty, "Unknown line")


    def data_available(self):
        """ Return true when there is more data available in the dump file."""
        #print("Data available: " + str(self._eof) + ", " + str(self._stop))
        try:
           self.get_size()
           self._currentPos = self._fileId.tell()
        except Exception as e:
           print(f'EchoReader: data_available exception {e}')
           pass

        self._eof = (self._currentPos == self._fileSize)
        ###return (self._eof == 0) and (self._stop == 0)
        ######print(f'data_available returning {not self._eof}')
        #return (self._eof) and (self._stop == 0)
        return not self._eof


    def decode_tc_type_subtype(self, t, s):
        try:
            return Euclid_TC_codes.Euclid_tc_types[(int(t), int(s))]
        except Exception as e:
            #print("EXCEPTION!")
            return "Unknown type, subtype (" + t + ", " + s + ")"


    def decode_tm_type_subtype(self, t, s):
        try:
            return Euclid_TM_codes.Euclid_tm_types[(int(t), int(s))]
        except Exception as e:
            return "Unknown type, subtype (" + t + ", " + s + ")"


    def reset(self):
       ###self._eof = 0
       self._eof = False
       self._stop = 0
