""" EddsReader.py module to read and decode the input csv file. It returns data in the
    same format as the EchoReader.py 
"""

import os, sys, re

#from Euclid import Euclid_TC_codes, Euclid_TM_codes
import Euclid_TC_codes, Euclid_TM_codes

""" This class provides an enum like constant for decoding the type of line read from the file. """
class LineType:
    TM, TC, Event, TM_PACKET, Empty, Err = range(6)

#logTime = ''

""" This class implements the csv file reader. """
class EddsReader(object):


    def __init__(self):
        super(EddsReader, self).__init__()

        self._filepath = ""
        self._ofilepath = ""
        #self._fileId = 0
        self._fileId = None
        self._ofileId = None
        self._line = ""
        self._error = 0
        self._eof = 0
        self._stop = 0
        self._logTime = ''


    ## Opens a cvs file for reading.
    #  @param filepath A string containing the full path to a cvs file.
    #  Read in and bin the first 3 header description lines
    def open_for_reading(self, filepath):
        """ Opens a cvs file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        #print("EddsReader Opening %s\n" % filepath)
        try:
            self._fileId = open(filepath, 'r')
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            for i in range(0, 3):
                foo = self.next_line()



    ## Opens a cvs file to write.
    #  @param filepath A string containing the full path to a cvs file.
    #  Read in and bin the first 3 header description lines
    def open_to_write(self, filepath, mode):
        """ Opens a cvs file to write.

        The parameter is the full path to an output file."""
        self._ofilepath = filepath + "_out"
        print("EddsReader Opening output file %s\n" % self._ofilepath)
        try:
            self._ofileId = open(self._ofilepath, 'a')
            if mode == 'realtime':
                self._ofileId.seek(0, 2)
            self._stop = 0
            self._eof = 0
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._ofilepath = filepath + "_out"
            #for i in range(0, 3):
                #foo = self.next_line()



    # Just in case the header lines are different from the csv file
    def open_csv_file(self, filepath, mode):
        """ Opens a csv file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        #Peter: EddsReader reads ex1.csv file
        print("EddsReader Opening %s\n" % filepath)
        try:
            self._fileId = open(filepath, 'r')
            if mode == 'realtime':
                self._fileId.seek(0, 2)
            self._stop = 0
            self._eof = 0
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            # Can't do this if interactive...
            #for i in range(0, 3):
                #foo = self.next_line()


    def close_input(self):
        """ Closes access to the cvs file. """
        if self._fileId == None:
            return
        try:
            self._fileId.close()
            self._ofileId.close()
            self._stop = 1
        except Exception as e:
            print("Error closing file")


    def stop(self):
        """ Module wide marker to implement the stop reading file function."""
        self._stop = 1

    
    def next_line(self):
        """ Return the next line from the dump file."""
        self._error = self._stop
#        if self._stop:
#            return self._stop
        try:
            self._line = self._fileId.readline().rstrip('\n')	# Works but screws up text field scrolling...
####            print("next_line: " + self._line)
        except Exception as e:
            print("Exception thrown reading file")
            self._line = "ERROR"
            self._error = 100
        finally:
            return self._error

#Events\tentryNo\tseverity\tsource\tworkstation\tobt\tlogTime\tmessage
#TM Parameters\tpk\tname\trawValue\tengValue\tmonState\tgenerationTime\trawValidity\tengValidity\tid\treceivedTime\tbitOffset\tbitSize\tsample\tcurve\tlimits\talert
#Commands\tname\tentryNo\tpktOffset\tpktSize\tid\tapid\tssc\ttype\tstype\tcompleted\ttimeTagged\tstageHistory\tmapid\tobt\ttrx\tlastUpdateTime\tparams

    def decode_line(self):
        #import pdb; pdb.set_trace()
        """ Return a string built from decoding the input line from the dump file."""
        search_pattern = re.compile("^TM Parameters")
        words = self._line.split('\t')
        if search_pattern.search(self._line):
            # TM, Generation time, name, rawValue, engValue, monState
            #print("decode_line 1", LineType.TM, (words[6],words[2],words[3],words[4],words[5]))
            return (LineType.TM, (words[6],words[2],words[3],words[4],words[5]))
        search_pattern = re.compile("^Events")
        if search_pattern.search(self._line):
            # Event, Generation time, log time, entryNo, severity, source, workstation, message...
            #print("decode_line 2", LineType.Event, words[6] + " Evt: " + words[1] + "/" + words[2] + " " + words[3] + " " +  words[4] + " ".join(words[7:]))
            return (LineType.Event, words[6] + " Evt: " + words[1] + "/" + words[2] + " " + words[3] + " " +  words[4] + " ".join(words[7:]))
        search_pattern = re.compile("^Commands")
        if search_pattern.search(self._line):
            type_subtype = self.decode_tc_type_subtype(words[8], words[9])
            # Command, lastupdateTime, ssc?
            #print("decode_line 3", LineType.TC, words[15] + " TC: " + words[6] + " (0x" + str(int(words[6], base=16)) + ") " + type_subtype)
            return (LineType.TC, words[15] + " TC: " + words[6] + " (0x" + str(int(words[6], base=16)) + ") " + type_subtype)

        self._eof = 1
        return (LineType.Err, "Unknown line")


# ICD 6.1.2.3 Event type=EV,entryNo/severity,source,category,logTime,domain
# ICD 6.1.3.1 TM header=logTime,length,ID,(type,stype),flag 
# ICD 6.7.3.3.2.1 defines TMPAR as follows
#  (1) name, string
#  (2) description, string is "TMPAR"
#  (3) unit, string
#  (4) type, string is "eng"
#  (5) active, bool, OK = True, or False  

    def decode_csv_line(self):
        """ Return a string built from decoding the input line from the dump file."""
        reject_pattern  = re.compile("name")    # TM, TMPAR
        reject_pattern1 = re.compile("logTime") # EV
        search_pattern = re.compile("^TMPAR")
        event_pattern = re.compile("^EV")
        packet_pattern = re.compile("^TM")

        words = self._line.split(',')
        
        if (reject_pattern.search(self._line)) or (reject_pattern1.search(self._line)):
            #self._eof = 1
            #print("REJECT")
            return (LineType.Err, "Unknown line")

        if search_pattern.search(self._line):
            #print('decode_csv_line 1 TMPAR', LineType.TM, ('logTime', words[1],words[2],words[3],words[4]))
            # TM, name (mib id), rawValue, engValue, monState
            #return (LineType.TM, ('logTime', words[1],words[2],words[3],words[4]))
            if words[4] == 'OK':
                return (LineType.TM, words[1] + ',TMPAR,' + words[3] + ',eng,True')
            else:
                return (LineType.TM, words[1] + ',TMPAR,' + words[3] + ',eng,False')
        
        search_pattern = re.compile("^EV")
        if search_pattern.search(self._line):
            #print('decode_csv_line 2 EV', LineType.Event, words[1] + " Evt: " + words[2] + "/" + words[3] + " " + words[4] + " " +  words[5] + " ".join(words[6:]))
            # Event, logTime, entryNo, severity, source, workstation, message
            #return (LineType.Event, words[1] + " Evt: " + words[2] + "/" + words[3] + " " + words[4] + " " +  words[5] + " ".join(words[6:]))
            #EV,entryNo/severity,source,message,logTime,workstation
            return (LineType.Event, "EV," + words[2] + "/" + words[3] + "," + words[4] + "," + " ".join(words[6:]) + "," + words[1] + "," + words[5])
        
        search_pattern = re.compile("^Commands")
        if search_pattern.search(self._line):
            #print('decode_csv_line 3 COMMAND', LineType.TC, words[15] + " TC: " + words[6] + " (0x" + hex(words[6]) + ") " + type_subtype)
            type_subtype = self.decode_tc_type_subtype(words[8], words[9]) 
            return (LineType.TC, words[15] + " TC: " + words[6] + " (0x" + hex(words[6]) + ") " + type_subtype)
        
        search_pattern = re.compile("^TM")
        if search_pattern.search(self._line):
            type_subtype = self.decode_tm_type_subtype(words[7], words[8])
            #logTime = words[11]
            self._logTime = words[11]
            # obt, apid, apid hex, type subtype string 
###            return (LineType.TM_PACKET, words[11] + " TM: " + words[5] + " " + type_subtype + ", " + words[9])
            #print('decode_csv_line 4 TM', LineType.TM_PACKET, (self._logTime, words[5], type_subtype, words[9]))
            return (LineType.TM_PACKET, (self._logTime, words[5], type_subtype, words[9], words[4]))
        
        self._eof = 1
        #return (LineType.Err, "Unknown line")
        return (LineType.Empty, "Unknown line")


    def data_available(self):
        """ Return true when there is more data available in the dump file."""
        #print("Data available: " + str(self._eof) + ", " + str(self._stop))
        return (self._eof == 0) and (self._stop == 0)


    def decode_tc_type_subtype(self, t, s):
        try:
            #print("decode_tc_type_subtype", Euclid_TC_codes.Euclid_tc_types[(int(t), int(s))])
            return Euclid_TC_codes.Euclid_tc_types[(int(t), int(s))]
        except Exception as e:
            #print("EXCEPTION!", "Unknown type, subtype (" + t + ", " + s + ")")
            return "Unknown type, subtype (" + t + ", " + s + ")"


    def decode_tm_type_subtype(self, t, s):
        try:
            #print("decode_tm_type_subtype", Euclid_TM_codes.Euclid_tm_types[(int(t), int(s))])
            return Euclid_TM_codes.Euclid_tm_types[(int(t), int(s))]
        except Exception as e:
            #print("Unknown type, subtype (" + t + ", " + s + ")")
            return "Unknown type, subtype (" + t + ", " + s + ")"


