""" 
This class creates and laysout a page of HK parameters and associated plot to be displayed in a tab.
"""

import sys
#import re

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick

from PyQt5.QtGui import QPainter, QBrush, QPen
from PyQt5.QtCore import Qt

from random import randint
from collections import deque

from pyqtgraph import PlotWidget, plot, ViewBox
import pyqtgraph as pg

import numpy as np
#import matplotlib.pyplot as plt

import datetime, time

from utils import TimeAxisItem, timestamp

###import iws_defs, iws, DisplayBox, EuclidHkBox

import GroupFrame
###import Graph

""" These constants refer to the offset of strings in the input feed from the data file """
MIB_NAME = 0
MIB_ID   = 1
HK_UNIT  = 2

#MAXLEN = 10000
MAXLEN = 1000

# json_data: [ {mib_name:mn,mib_id:mi,...}, {} ]
class GraphFrame(QtWidgets.QFrame):

    """
    json_data = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v}
    json = {"name":n,"naxes":n,"yaxis1":{},"yaxis2:{},"labels:[]}
    """
    def __init__(self, name, json_data, parent=None, legend=True):
        super(GraphFrame, self).__init__()

#        self.graph = Graph.Graph(name, json_data, self, legend)

        self.name = name
        self.legend = legend
        self.labels = {}

        # Use this array in updating plots to do array processing instead of iteration
        self.label_array = []

        # x data values to be plotted
        self.x_data = deque([0.0], maxlen = MAXLEN)
        #print(f'GraphFrame init: x_data dequeue len = {len(self.x_data)}')
###        self.x_data = np.zeros(MAXLEN, dtype = int)

        # Dictionary of y data deques, indexed by mib_name string
        self.y_data = {}

        # pyqtgraph references to each parameter's line plot
        self.y_data_points = {}

        # Plot line colours
        self.pens = [(255,0,0),(0,255,0),(0,0,255),(0,0,0),(255,255,0),(0,255,255),
                     (255,0,255),(255,0,128),(255,128,0),(0,255,128),(128,255,0),(0,128,255)]

        # Dictionary for each parameter's plot colour
        self.y_colours = {}

        # Holds the latest engineering value for each parameter
        self.y_vals = {}

        self.x = 0

        self.last_update_obt = datetime.datetime.utcnow()
        
        left_frame  = self.create_left_frame(json_data)
        right_frame = QtWidgets.QFrame()

        graph_frame = QtWidgets.QFrame()
        #graph_frame.setStyleSheet("background-color:lightgreen")

        # Create plot
        yaxis_label = json_data['yaxis1']['label']
        ymin = json_data['yaxis1']['min']
        ymax = json_data['yaxis1']['max']

        plot_frame = QtWidgets.QFrame()
        #plot_frame.setStyleSheet("background-color:green")
        
        #self.graphWidget = pg.PlotWidget()
        #axis = pg.DateAxisItem()
        #self.graphWidget.setAxisItems({'bottom':axis})

        '''
        plotItem = pg.PlotItem(axisItems={'bottom': pg.DateAxisItem(orientation='bottom')})
        self.graphWidget = pg.PlotWidget(plotItem = plotItem)
        '''
        ###self.graphWidget = pg.PlotWidget(axisItems={'bottom': TimeAxisItem(orientation='bottom')})
        self.graphWidget = pg.PlotWidget(axisItems={'bottom': pg.DateAxisItem(orientation='bottom')})
        ####self.graphWidget = pg.PlotItem(axisItems={'bottom': pg.DateAxisItem(orientation='bottom')})

###        self.graphWidget = pg.PlotWidget()
###        self.graphWidget.setFixedHeight(300)

        self.graphWidget.setFixedHeight(350)
        self.graphWidget.setBackground('w')

        #self.graphWidget.setTitle(json_data['name'], color='b', size='150')
        #self.graphWidget.setLabel('left', 'Temperature (C)', color='red', size=30)
        #self.graphWidget.setLabel('left', yaxis_label, color='red', size=30)
        #self.graphWidget.setLabel('bottom', 'Time', color='red', size=30)

        self.graphWidget.setTitle(json_data['name'], size='150')
        #self.graphWidget.setLabel('left', 'Temperature (C)', size=30)
        self.graphWidget.setLabel('left', yaxis_label, size=30)
        self.graphWidget.setLabel('bottom', 'Time', size=30)

        #self.graphWidget.addLegend()
        self.graphWidget.showGrid(x=True, y=True)
        self.graphWidget.setYRange(ymin, ymax, padding=0)

        self.graphWidget.enableAutoRange(axis=ViewBox.YAxis, enable=True)

        #self.graphWidget.addLegend()

        if not self.legend:
            self.graphWidget.addLegend(offset=(1,1), verSpacing = -10, labelTextSize = '6pt')
            #legendLabelStyle = {'size': '6pt', 'bold': True, 'italic': False}
            #for item in self.graphWidget.legend.items:
                #for single_item in item:
                    #if isinstance(single_item, pg.graphicsItems.LabelItem.LabelItem):
                        #single_item.setText(single_item.text, **legendLabelStyle)

        # Set up plot parameters
        count = 0
        for name in self.labels:
            self.y_data[name] = deque([0.0], maxlen = MAXLEN)
###            self.y_data[name] = np.zeros(MAXLEN, dtype = float)
            self.y_vals[name] = 0.0
            color=self.pens[count]
            pen = pg.mkPen(color=color)
            self.y_data_points[name] = self.graphWidget.plot(name = self.labels[name].name, pen=pen, symbol = 'o', symbolSize=2, symbolBrush=(color), symbolPen = pen)
            count += 1
            #self.graphWidget.setAutoPan(y=self.y_data[name])

        """
        graph_frame_layout = QtWidgets.QHBoxLayout()
        graph_frame_layout.setContentsMargins(0, 0, 0, 0)
        #graph_frame_layout.addWidget(labels_frame)
        graph_frame_layout.addWidget(self.graphWidget, stretch = 1)
        graph_frame.setLayout(graph_frame_layout)
        """
        
        plot_frame_layout = QtWidgets.QHBoxLayout()
        plot_frame_layout.setContentsMargins(0, 0, 0, 0)
        plot_frame_layout.addWidget(self.graphWidget, stretch = 1) ### Original
        plot_frame.setLayout(plot_frame_layout)

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        if self.legend:
            layout.addWidget(left_frame)
        ###layout.addWidget(graph_frame, stretch = 1)
        #layout.addWidget(self.graphWidget, stretch = 1)
        layout.addWidget(plot_frame, stretch = 1) # This forces the left_frame to the left

#        layout = QtWidgets.QHBoxLayout() # temp
#        layout.setContentsMargins(0, 0, 0, 0) # temp
#        layout.addWidget(self.graph, stretch = 1) # This forces the left_frame to the left

        self.setLayout(layout)

        ########labels = self.group.parameters() ####!!!

        if parent != None:
            parent.ui.all_plot_labels.update(self.labels)
            self.label_array.append({k:self.labels[k] for k in self.labels})
            #print(f'GraphFrame label_array = {self.label_array}')


    def debug(self):
        print("Tab " + self.name)
        for label in self.labels:
            self.labels[label].debug()


    def create_check_box_frame(self):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:lightblue")

        font = self.font()
        font.setBold(False)
        self.setFont(font)

        self.checkbox = QtWidgets.QCheckBox("Engineering")
        self.checkbox.setObjectName("checkbox")
        self.checkbox.setChecked(True)
        self.checkbox.toggled.connect(self.engineering_check_box_clicked)
        self.checkbox.setDisabled(True)

        self.mib_checkbox = QtWidgets.QCheckBox("Use MIB name")
        self.mib_checkbox.setObjectName("mib_checkbox")
        self.mib_checkbox.setChecked(True)
        self.mib_checkbox.toggled.connect(self.mib_check_box_clicked)

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.checkbox)
        layout.addWidget(self.mib_checkbox, stretch = 1) # This forces the boxes to the left
        frame.setLayout(layout)
        return frame


    def parameters(self):
        return self.labels

    
    def create_labels_frame(self, json_data):
        frame = GroupFrame.GroupFrame(json_data['labels'])
        #frame.setStyleSheet("background-color:purple")
        self.group = frame
        self.labels.update(self.group.parameters())
        return frame

        
    def create_labels_key_frame(self):
        frame = QtWidgets.QFrame()
        frame.setMinimumSize(QtCore.QSize(20, 20))
        #frame.setStyleSheet("background-color:red")
        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        count = 0
        font = QtGui.QFont()
        font.setPointSize(6)
        for label in self.labels:
            color = self.pens[count]
            count += 1
            r = str(color[0])
            g = str(color[1])
            b = str(color[2])
            rgb = f"rgb({r},{g},{b})"
            l = QtWidgets.QLabel("Line")
            l.setFont(font)
            l.setStyleSheet(f"background-color:{rgb};color:{rgb}")
            layout.addWidget(l)
        frame.setLayout(layout)
        return frame


    def create_middle_frame(self, json_data):
        frame = QtWidgets.QFrame()
        labels_frame = self.create_labels_frame(json_data)
        #print(f'GraphFrame create_middle_frame: labels = {self.labels}')
        labels_key_frame = self.create_labels_key_frame()

        layout = QtWidgets.QHBoxLayout()

        layout.addWidget(labels_key_frame)
        layout.addWidget(labels_frame)
        #layout.addStretch(1)
        ##########layout.addWidget(labels_key_frame)
        #layout.addStretch(1)
        frame.setLayout(layout)
        return frame


    def create_left_frame(self, json_data):
        frame = QtWidgets.QFrame()
        #frame.setStyleSheet("background-color:pink")
        check_box_frame = self.create_check_box_frame()
        middle_frame = self.create_middle_frame(json_data)

        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(5, 0, 0, 0)
        layout.setSpacing(0)
        layout.addWidget(check_box_frame)
        layout.addWidget(middle_frame)
        layout.addStretch(1) # This forces the check_box_frame to the top
        frame.setLayout(layout)
        return frame


    def parameters(self):
        return self.group.parameters()


    def engineering_check_box_clicked(self):
        #print("Engineering checkbox clicked")
        if self.checkbox.isChecked():
            self.group.display_engineering()
        else:
            self.group.display_raw()


    def mib_check_box_clicked(self):
        #print("Mib checkbox clicked")
        if self.mib_checkbox.isChecked():
            self.group.display_mib_name()
        else:
            self.group.display_mib_id()


    def update_tm(self, parameter_string):
        self.group.update_tm(parameter_string)


    def clear_plot_data(self):
        self.x_data.clear()
        self.x = 0
        self.x_data = deque([0.0], maxlen = MAXLEN)
        for label in self.labels:
            self.y_data[label].clear()
            self.y_data[label] = deque([0.0], maxlen = MAXLEN)
        print("Finished Clearing")


    def update_plot_lastentry(self):
        update_plot(self.last_update_obt)


    # obt_time is a string
    def update_plot(self, obt_time, time_conversion_string, timeval):
###        pass

        #print("OBT Time: " + str(obt_time))
        self.last_update_obt = obt_time
        
        #get_time = datetime.datetime.strptime(obt_time, time_conversion_string)

        #xaxis_time_str = str(get_time.hour) + ":" + str(get_time.minute) + ":" + str(get_time.second)
        #xval = int(time.mktime(get_time.timetuple()))
        xval = timeval

        if self.x == 0:
            self.x_data[self.x] = xval
        else:
            self.x_data.append(xval)

        # Access everythin as arrays and use array update, no iteration
        #self.y_data[:][self.x] = self.group.parameters()

        #new_y_data = []

        for label in self.labels:
            eng_val = self.group.eng_val(label)
            if self.x == 0:
                #self.y_data[name][self.x] = self.y_vals[name]
                self.y_data[label][self.x] = eng_val
            else:
                #self.y_data[name].append(self.y_vals[name])
                self.y_data[label].append(eng_val)
                #self.y_data[label][self.x] = eng_val

            #new_y_data.append(eng_val)

            try:
                self.y_data_points[label].setData(self.x_data, self.y_data[label])
            except Exception as ex:
                print(f"Exception '{ex}' thrown in GraphFrame:update_plot")
                print(f"x: {self.x} < label: {label}, eng_val = {eng_val}")
                print(f'{self.y_data_points[label]}.setData({self.x_data}, {self.y_data[label]}')
            else:
                pass

        #self.y_data_points[self.labels].setData(self.x_data, new_y_data)

        self.x += 1


        """
        if self.x < MAXLEN:
            if self.x == 0:
                ###self.x_data[self.x] = self.x
                self.x_data[self.x] = xval
            else:
                ###self.x_data.append(self.x)
                self.x_data.append(xval)
###                self.x_data[self.x] = xval
        else:
            self.x_data.rotate(-1)
            ###self.x_data[-1] = self.x
            self.x_data[-1] = xval

        for label in self.labels:
            eng_val = self.group.eng_val(label)
            if self.x < MAXLEN:
                if self.x == 0:
                    #self.y_data[name][self.x] = self.y_vals[name]
                    self.y_data[label][self.x] = eng_val
                else:
                    #self.y_data[name].append(self.y_vals[name])
                    self.y_data[label].append(eng_val)
###                    self.y_data[label][self.x] = eng_val
            else:
                self.y_data[label].rotate(-1)
                #self.y_data[name][-1] = self.y_vals[name]
                self.y_data[label][-1] = eng_val
            try:
                self.y_data_points[label].setData(self.x_data, self.y_data[label])
            except Exception as ex:
                print(f"Exception '{ex}' thrown in GraphFrame:update_plot")
                print(f"x: {self.x} < label: {label}")
            else:
                pass

        self.x += 1
        """

        #test to see what the graph does
        #nope not great
        #if self.x % 20  == 0:
        #    self.x += 22
        #self.tm_updated = False
        #print("Finished update_plot")



    #def update_display1(self, raw_val_str, eng_val_str, eng_val_tuple, status):
        #pass
 

    """
    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)
        count = 0
        x_origin = 100
        y_origin = 100
        delta_y  = 20
        # Set up plot parameters
        for label in self.labels:
            #name = label[MIB_ID]
            #self.y_data[name] = deque([0.0])
            #self.y_vals[name] = 0.0
            color=self.pens[count]
            pen = pg.mkPen(color=color)
            painter.setPen(pen)
            #painter.setPen(QPen(pen, 4, Qt.SolidLine))
            painter.setBrush(Qt.white)
            ###(x,y) = DisplayBox.label_key_origin(count)
            x = x_origin
            y = y_origin
            #print("x = " + str(x) + ", y = " + str(y))
            painter.drawLine(x, y, x + 60, y)
            count += 1
            y_origin += delta_y
        painter.end()
    """
