#!/usr/bin/env python

import os

from PyQt5 import QtWidgets, QtGui, QtCore    #, uic, QtQml, QtQuick
from PyQt5.QtWidgets import QFileDialog, QMessageBox
from PyQt5.QtCore import Qt, QTimer

import sys
import time
from time import sleep

import configparser

import EchoReader
import iws_defs

#from Configuration_dialog import Configuration_Dialog as Form
from Dialog import Ui_Dialog as Form

from IWS_MainWindow import IWS_MainWindow

""" The top level window of the pogram. """
class mywindow(QtWidgets.QMainWindow):
 
    def __init__(self):
        super(mywindow, self).__init__()

        self.watcher = None
        #self.source = os.environ['HOME'] + "/Projects/Euclid/DART_dumps/TM_dump3.tsv"
        self.source = os.environ['HOME'] + "/Projects/Euclid/DART_dumps/CEA_Day3_3.25.csv"
        self.file_format = ''
        self.dump_dir = ""

        self.reader = EchoReader.EchoReader()

        self.output_lines = 0
        #self.ui.textEdit.verticalScrollBar().setMaximum(self.output_lines)

        self._do_plot = 0

        #self.timer = QTimer(self)
        #self.timer.timeout.connect(self.update)
        #self.timer.setSingleShot(False)

        #self.debug()
        
        #self.connect(self, SIGNAL("sendValue(PyQt_PyObject)"), self.line_read)

    #def debug(self):
        #print("Main debug\n")
        #for key in self.parameters:
            #print("Key = [" + key + "]\n")
        #print("Main debug finished\n")

    def layout(self, config_file):
        self.ui = IWS_MainWindow()

        #print("CONFIG: [" + iws_defs.default_config + "]")
        self.ui.setupUi(self, config_file)

        self.ui.pushButton_start.clicked.connect(self.startClicked) # connecting the clicked signal with btnClicked slot
        self.ui.pushButton_stop.clicked.connect(self.stopClicked)
        self.ui.pushButton_quit.clicked.connect(self.quitClicked)
        

    def open_dialog(self, config_file):
        self.dialog = QtWidgets.QDialog()
        self.dialog.ui = Form()
        self.dialog.ui.setupUi(self.dialog, config_file)

        if self.dialog.exec_():
            print("Dialog done (OK)")
            #dialog.show()
        else:
            print("Dialog done (Cancel)")

        
    def line_read(self, value):
        pass

    
    # Temporary, should be in event handler for file modified
    def update(self):
        #print("Update")
        #while self.reader.data_available():
        result = self.reader.next_line()
        if result != 0:
            return
            #break
        self.reader.decode_line()
        (lineType, string) = self.reader.decode_line()
        if lineType == EchoReader.LineType.Event:
            self.ui.eventText.appendPlainText(string)
        elif lineType == EchoReader.LineType.TC:
            self.ui.tcText.appendPlainText(string)
        elif lineType == EchoReader.LineType.TM:
            self.ui.update_tm(string)
        else:
            return
            #break
        QtWidgets.QApplication.processEvents()
# end while
        #print(self.ui.eventText.verticalScrollBar().value())


    # Temporary
    def update1(self):
        while self.reader.data_available():
            result = self.reader.next_line()
            if result != 0:
                return
            #break
            #self.reader.decode_line()
            #(lineType, string) = self.reader.decode_line()
            (lineType, string) = self.reader.decode_csv_line()
            #print("Linetype:")
            #print(lineType)
            #print("String:")
            #print(string)
            #print(str(lineType) + ' : ' + string)
            if lineType == EchoReader.LineType.Event:
                self.ui.eventText.appendPlainText(string)
            elif lineType == EchoReader.LineType.TC:
                self.ui.tcText.appendPlainText(string)
            elif lineType == EchoReader.LineType.TM:
                self.ui.update_tm(string)
            elif lineType == EchoReader.LineType.TM_PACKET:
                #self.ui.update_tm(string)
                #print("TM PACKET")
                pass
            else:
                return
            #break
            QtWidgets.QApplication.processEvents()
        # end while
            #print(self.ui.eventText.verticalScrollBar().value())


    def csv_update(self):
        #print('csv_update')
        while self.reader.data_available():
            result = self.reader.next_line()
            if result != 0:
                print('cvs_update: bad read')
                return
            #self.reader.decode_line()
            #(lineType, string) = self.reader.decode_line()
            (lineType, string) = self.reader.decode_csv_line()
            
            #print('csv_update in while loop')
            #print("Linetype:")
            #print(lineType)
            #print("String:")
            #print(string)
            #self.ui.eventText.appendPlainText(string)
            
            if lineType == EchoReader.LineType.Event:
                self.ui.eventText.appendPlainText(string)
            elif lineType == EchoReader.LineType.TC:
                self.ui.tcText.appendPlainText(string)
            elif lineType == EchoReader.LineType.TM:
                #self.ui.tcText.appendPlainText(string)
                self.ui.update_tm(string)
                #pass
            elif lineType == EchoReader.LineType.TM_PACKET:
                if self._do_plot == 1:
                    self.ui.update_plots()
                else:
                    self._do_plot = 1
                self.ui.tcText.appendPlainText(string)
                #self.ui.update_tm(string)
                #print("TM PACKET")
            elif lineType == EchoReader.LineType.Empty:
                if self._do_plot == 1:
                    self.ui.update_plots()
                    self._do_plot = 0
                else:
                    pass
            else:
                pass
                #return
            
            QtWidgets.QApplication.processEvents()
        sleep(0.05)
        # end while
        self.reader._eof = 0
            #print(self.ui.eventText.verticalScrollBar().value())
            

    def read_ini(self):
        try:
            cfg = configparser.ConfigParser()
            cfg.read(iws_defs.ini_file)
            self.dump_dir = cfg['SOURCE']['dump_dir']
        except  Exception as e:
            pass


    def open_source(self):
        # Open config.ini and read source directory
        self.read_ini()

        # Present file open dialog
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        #filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", "", "All Files (*);;csv files (*.csv);;tsv files (*.tv)", options=options)
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", self.dump_dir, "All Files (*);;csv files (*.csv);;tsv files (*.tsv)", options=options)
        # If file selected open file for reading & start timer
        if filename:
            #print(filename)
            self.source = filename
            self.ui.file_textbox.setText(self.source)
#            print("Watching " + self.source)
            return 1
        return 0


    def quitClicked(self):
        quitbuttonReply = QMessageBox.question(self,
                                               'PyQt5 message', "Are you sure you want to quit the HK/TM monitor?",
                                               QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if quitbuttonReply == QMessageBox.Yes:
            print('Yes clicked. Exiting ...')
            # Write back to config.ini file
            try:
                cfg = configparser.ConfigParser()
                path = os.path.dirname(self.source)
                print("Saving " + path + " to ini file at " + iws_defs.ini_file)
                cfg['SOURCE'] = {'dump_dir' : path}
                with open(iws_defs.ini_file, 'w') as ini_file:
                    cfg.write(ini_file)
            except  Exception as e:
                print('Failed to write to ini file')
                print(e)
            self.close()
            exit(-2)
        else:
            print('No clicked.')
        return


    def stopClicked(self):
        #print("Stop clicked")
        #self.timer.stop()
        self.reader.stop()
        self.reader.close_input()


    def startClicked(self):
        #print("Start button clicked")

        if self.open_source():

        #self.reader.open_for_reading(os.environ['HOME'] + "/Projects/Euclid/DART_dumps/TM_dump3.tsv")
        #self.reader.open_for_reading(self.source)
            self.reader.open_csv_file(self.source)
            self.watcher = QtCore.QFileSystemWatcher([self.source])
            #self.watcher.connect(self.watcher, QtCore.SIGNAL('fileChanged(QString)'), self, file_changed)
            self.watcher.fileChanged.connect(self.file_changed)
            watched_files = self.watcher.files()
            for f in watched_files:
                print("Watched file: " + f)
            #self.csv_update()


    def file_changed(self):
        #print("File changed")
        sleep(0.05)
        self.reader._eof = 0
        #result = self.reader.next_line()
        #print("Got " + self.reader._line)
        self.csv_update()


app = QtWidgets.QApplication([])
application = mywindow()
application.open_dialog(iws_defs.config_file)
config = iws_defs.config
application.layout(iws_defs.defaults_directory + config)

#print("application.show()")
application.show()

#print("sys.exit(app.exec_())")
sys.exit(app.exec_())
#print("Sys exit")



#app = QtWidgets.QApplication([])
#win = uic.loadUi("hello.ui") #specify the location of your .ui file
#win.show()
#sys.exit(app.exec())
